import pytest

import httpretty

from httpretty import httprettified
from opencage.geocoder import OpenCageGeocode
from opencage.geocoder import UnknownError

geocoder = OpenCageGeocode('abcde')

@httprettified
def test_http_500_status():
    httpretty.register_uri(
        httpretty.GET,
        geocoder.url,
        status=500,
    )

    with pytest.raises(UnknownError) as excinfo:
        geocoder.geocode('whatever')

    assert str(excinfo.value) == '500 status code from API'

@httprettified
def test_non_json():
    "These kinds of errors come from webserver and may not be JSON"
    httpretty.register_uri(
        httpretty.GET,
        geocoder.url,
        body='<html><body><h1>503 Service Unavailable</h1></body></html>',
        forcing_headers={
            'Content-Type': 'text/html',
        },
        status=503
    )

    with pytest.raises(UnknownError) as excinfo:
        geocoder.geocode('whatever')

    assert str(excinfo.value) == 'Non-JSON result from server'

@httprettified
def test_no_results_key():
    httpretty.register_uri(
        httpretty.GET,
        geocoder.url,
        body='{"spam": "eggs"}',
    )

    with pytest.raises(UnknownError) as excinfo:
        geocoder.geocode('whatever')

    assert str(excinfo.value) == "JSON from API doesn't have a 'results' key"



# encoding: utf-8


from opencage.geocoder import OpenCageGeocode

# Check if protocol can be set
geocoder = OpenCageGeocode('abcde', 'http')
assert geocoder.url == 'http://api.opencagedata.com/geocode/v1/json'



# encoding: utf-8

from pathlib import Path

import os
import re
import httpretty

from httpretty import httprettified
from opencage.geocoder import OpenCageGeocode

# reduce maximum backoff retry time from 120s to 1s
os.environ['BACKOFF_MAX_TIME'] = '1'

geocoder = OpenCageGeocode('abcde', user_agent_comment='OpenCage Test')

user_agent_format = re.compile(r'^opencage-python/[\d\.]+ Python/[\d\.]+ (requests|aiohttp)/[\d\.]+ \(OpenCage Test\)$')

@httprettified
def test_sync():
    httpretty.register_uri(
        httpretty.GET,
        geocoder.url,
        body=Path('test/fixtures/uk_postcode.json').read_text(encoding="utf-8")
    )

    geocoder.geocode("EC1M 5RF")
    user_agent = httpretty.last_request().headers['User-Agent']

    assert user_agent_format.match(user_agent) is not None



import pytest

import httpretty

from httpretty import httprettified
from opencage.geocoder import OpenCageGeocode
from opencage.geocoder import InvalidInputError

geocoder = OpenCageGeocode('abcde')

@httprettified
def test_must_be_unicode_string():
    httpretty.register_uri(
        httpretty.GET,
        geocoder.url,
        body='{"results":{}}'
    )

    # Should not give errors
    geocoder.geocode('xxx')   # ascii convertable
    geocoder.geocode('xxá')   # unicode

    # But if it isn't a unicode string, it should give error
    utf8_string = "xxá".encode("utf-8")
    latin1_string = "xxá".encode("latin1")

    with pytest.raises(InvalidInputError) as excinfo:
        geocoder.geocode(utf8_string)
    assert str(excinfo.value) == f"Input must be a unicode string, not {utf8_string!r}"
    assert excinfo.value.bad_value == utf8_string

    with pytest.raises(InvalidInputError) as excinfo:
        geocoder.geocode(latin1_string)
    assert str(excinfo.value) == f"Input must be a unicode string, not {latin1_string!r}"
    assert excinfo.value.bad_value == latin1_string
